/* Steven Stanek
 * April 9, 2001
 * YZChess  Main.c	
 */


#include "yzchess.h"


picture_t loadPictures();
square_t** initBoard(picture_t pics);
void drawBoard(square_t **dBoard);
void drawSquare(square_t **dBoard, int x_num, int y_num); //NUM Squares, not pixels
void handleClicks(square_t **dBoard, picture_t pics);
bool isValidMove(square_t **dBoard, move_t pres_move, move_t last_move, picture_t pics, bool allowspecial);
bool isValidPawnMove(square_t **dBoard, move_t pres_move, move_t last_move, picture_t pics, bool allowenpass);
bool isValidRookMove(square_t **dBoard, move_t move);
bool isValidKnightMove(move_t move);
bool isValidBishopMove(square_t **dBoard, move_t move);
bool isValidQueenMove(square_t **dBoard, move_t move);
bool isValidKingMove(square_t **dBoard, move_t move, bool allowcastle);
bool isCheck(square_t **dBoard, bool isWhite);
bool moveChecks(square_t **dBoard, bool isWhite, move_t move);
bool isMate(square_t **dBoard, bool isWhite);
void getKingCoords(square_t **dBoard, bool isWhite, int *x, int *y);
bool mateKingMove(square_t **dBoard, bool isWhite, int king_x, int king_y);
bool mateOtherMove(square_t **dBoard, bool isWhite);
bool blockPossible(square_t **dBoard, bool isWhite, int block_x, int block_y);
square_t **dupBoard(square_t **dBoard);
void delBoard(square_t **tBoard);

main()
{
	window_t *dWindow;
	square_t **dBoard;
	picture_t pics;
	
	dWindow = yzNewWindow("YZChess", WINDOW_WIDTH, WINDOW_HEIGHT);
	yzSelectWindow(dWindow);
	pics = loadPictures();
	dBoard=initBoard(pics);
	drawImage(pics.exitbutton, 0, BOTTOM_START);
	drawImage(pics.about, getImageWidth(pics.exitbutton), BOTTOM_START);
	
	handleClicks(dBoard, pics);
}


picture_t loadPictures()
{
	picture_t pics;
	FILE *in;
	
	assert(in = fopen(PIECES_FN, "r"));
	
	pics.bpawn = openImageFile(in);
	pics.wpawn = openImageFile(in);
	pics.brook = openImageFile(in);
	pics.wrook = openImageFile(in);
	pics.bknight = openImageFile(in);
	pics.wknight = openImageFile(in);
	pics.bbishop = openImageFile(in);
	pics.wbishop = openImageFile(in);
	pics.bqueen = openImageFile(in);
	pics.wqueen =openImageFile(in);
	pics.bking = openImageFile(in);
	pics.wking = openImageFile(in);
	pics.exitbutton = openImageFile(in);
	pics.about = openImageFile(in);
	pics.blackmate = openImageFile(in);
	pics.whitemate = openImageFile(in);
	
	fclose(in);
	
	return pics;	
}


square_t** initBoard(picture_t pics)
{
	int i, j;
	square_t ** dBoard;
	
	assert(dBoard = malloc(NUM_SQUARES_TALL * sizeof(square_t*)));
	for(i=0; i < NUM_SQUARES_WIDE; i++)
		assert(dBoard[i]=malloc(NUM_SQUARES_WIDE * sizeof(square_t)));
	
	//Set Square Colors
	for(i=0; i < NUM_SQUARES_TALL; i++)
		for(j=0; j < NUM_SQUARES_WIDE; j++)
		{
			dBoard[i][j].isWhite= (i%2) ? (j) % 2: (j+1) % 2;
			dBoard[i][j].isSelected = false;
		}
	
	//init pawns
	for(i=0; i < NUM_SQUARES_WIDE; i++)
	{
		dBoard[2][i].contains = dBoard[3][i].contains = dBoard[4][i].contains = dBoard[5][i].contains = 0;
		
		//Create Pieces in starting squares
		assert(dBoard[0][i].contains = malloc(sizeof(piece_t)));
		assert(dBoard[1][i].contains = malloc(sizeof(piece_t)));
		assert(dBoard[6][i].contains = malloc(sizeof(piece_t)));
		assert(dBoard[7][i].contains = malloc(sizeof(piece_t)));
		
		//Set Piece Colors
		dBoard[0][i].contains->isWhite=dBoard[1][i].contains->isWhite = false;
		dBoard[7][i].contains->isWhite=dBoard[6][i].contains->isWhite = true;
		
		//Set Piece types for Pawns only
		dBoard[6][i].contains->type=dBoard[1][i].contains->type = pawn;
		
		//add line for loading image
		dBoard[1][i].contains->pic = pics.bpawn;
		dBoard[6][i].contains->pic = pics.wpawn;
	}
	
	//Set pieces
	dBoard[0][0].contains->type=dBoard[0][7].contains->type=dBoard[7][0].contains->type=dBoard[7][7].contains->type=rook;
	dBoard[0][1].contains->type=dBoard[0][6].contains->type=dBoard[7][1].contains->type=dBoard[7][6].contains->type=knight;
	dBoard[0][2].contains->type=dBoard[0][5].contains->type=dBoard[7][2].contains->type=dBoard[7][5].contains->type=bishop;
	dBoard[0][3].contains->type=dBoard[7][3].contains->type=queen;
	dBoard[0][4].contains->type=dBoard[7][4].contains->type=king;
	
	//pic loaders
	dBoard[0][0].contains->pic = dBoard[0][7].contains->pic = pics.brook;
	dBoard[7][0].contains->pic = dBoard[7][7].contains->pic = pics.wrook;
	dBoard[0][1].contains->pic = dBoard[0][6].contains->pic = pics.bknight;
	dBoard[7][1].contains->pic = dBoard[7][6].contains->pic = pics.wknight;
	dBoard[0][2].contains->pic = dBoard[0][5].contains->pic = pics.bbishop;
	dBoard[7][2].contains->pic = dBoard[7][5].contains->pic = pics.wbishop;
	dBoard[0][3].contains->pic = pics.bqueen;
	dBoard[7][3].contains->pic = pics.wqueen;
	dBoard[0][4].contains->pic = pics.bking;
	dBoard[7][4].contains->pic = pics.wking;
	
	drawBoard(dBoard);
	return dBoard;
}


void drawBoard(square_t **dBoard)
{
	int i, j;
	
	for(i=0; i < NUM_SQUARES_TALL; i++)
	{
		for(j=0; j < NUM_SQUARES_WIDE; j++)
			drawSquare(dBoard, j, i);
	}
}


void drawSquare(square_t **dBoard, int x_num, int y_num)
{
	int x, y;
	
	x = x_num * PIECE_SIDE;
	y = y_num * PIECE_SIDE;
		
	//Draw BG Color for square
	if(dBoard[y_num][x_num].isWhite)
			yzPenColor(65535,65535,65535);
	else
			yzPenColor(GRAY_LEVEL, GRAY_LEVEL, GRAY_LEVEL);	
	
	yzRect(x, y, x + PIECE_SIDE, y + PIECE_SIDE);
	
	//Draw Image if it exists
	if(dBoard[y_num][x_num].contains)
		drawImage(dBoard[y_num][x_num].contains->pic,x,y);
	
	//Draw Border
	yzPenColor(0,0,0);
	yzLine(x,y, x, y+PIECE_SIDE);
	yzLine(x,y, x+PIECE_SIDE, y);
	yzLine(x+PIECE_SIDE, y+PIECE_SIDE, x, y+PIECE_SIDE);
	yzLine(x+PIECE_SIDE, y+PIECE_SIDE, x+PIECE_SIDE, y);
	
	if(dBoard[y_num][x_num].isSelected)
	{
		yzPenColor(0,0,0);
		yzLine(x+1,y+1, x+1, y+PIECE_SIDE-1);
		yzLine(x+1,y+1, x+PIECE_SIDE-1, y+1);
		yzLine(x+PIECE_SIDE-1, y+PIECE_SIDE-1, x+1, y+PIECE_SIDE-1);
		yzLine(x+PIECE_SIDE-1, y+PIECE_SIDE-1, x+PIECE_SIDE-1, y+1);
	}
}


void handleClicks(square_t **dBoard, picture_t pics)
{
	int x, y, query;
	move_t presMove, lastMove;
	bool whiteTurn = true;
	
	lastMove.start_x = -1;
	
	while(1)
	{
		do
		{
			query=yzQueryPointer(&x, &y);
			
			if(query == 3 && x > 0 && x < getImageWidth(pics.exitbutton) && y > BOTTOM_START)
				return;
		} while (query !=3);
	
		presMove.start_x = x / PIECE_SIDE;
		presMove.start_y = y / PIECE_SIDE;
			
		if(presMove.start_x < NUM_SQUARES_WIDE && presMove.start_y < NUM_SQUARES_TALL && 
			dBoard[presMove.start_y][presMove.start_x].contains && 
			dBoard[presMove.start_y][presMove.start_x].contains->isWhite == whiteTurn)
		{
			//Draw Selected Start Square
			dBoard[presMove.start_y][presMove.start_x].isSelected = true;
			drawSquare(dBoard, presMove.start_x, presMove.start_y);
		
			// Query for Selection of end square
			do
			{
				query=yzQueryPointer(&x, &y);
			} while (query !=3);
		
			presMove.end_x = x / PIECE_SIDE;
			presMove.end_y = y / PIECE_SIDE;
		
			if(presMove.end_x < NUM_SQUARES_WIDE && presMove.end_y < NUM_SQUARES_TALL &&
				isValidMove(dBoard, presMove, lastMove, pics, true) && !moveChecks(dBoard, whiteTurn, presMove))
			{
				// Process Click - rewrite squares, redraw, Flip Turns, set last move
				dBoard[presMove.end_y][presMove.end_x].contains = dBoard[presMove.start_y][presMove.start_x].contains;
				dBoard[presMove.start_y][presMove.start_x].contains =0;
				
				whiteTurn = whiteTurn ? false : true;
				lastMove=presMove;
			}
		
			dBoard[presMove.start_y][presMove.start_x].isSelected = false;
			drawSquare(dBoard, presMove.start_x, presMove.start_y);
			drawSquare(dBoard, presMove.end_x, presMove.end_y);
			
			if(isMate(dBoard, whiteTurn))
			{
				whiteTurn = !whiteTurn;
				if(whiteTurn)	
					drawImage(pics.whitemate, WINDOW_WIDTH /2 - getImageWidth(pics.whitemate) /2, 
						          (BOTTOM_START /2 - getImageHeight(pics.whitemate)/2));
				else
					drawImage(pics.blackmate, WINDOW_WIDTH /2 - getImageWidth(pics.blackmate) /2, 
					          (BOTTOM_START /2 - getImageHeight(pics.blackmate)/2));
				while(yzQueryPointer(&x, &y) !=3)
				{
				}
				return;
			}
		}
	}
}

//If last_move.start_x = -1 this is first move, does not check check
bool isValidMove(square_t **dBoard, move_t pres_move, move_t last_move, picture_t pics, bool allowspecial)
{
	//Check that pres_move is inbounds
	if(pres_move.start_x < 0 || pres_move.start_x  >= NUM_SQUARES_WIDE || pres_move.start_y < 0 || 
	   pres_move.start_y >= NUM_SQUARES_TALL || pres_move.end_x < 0 || pres_move.end_x >= NUM_SQUARES_WIDE ||
	   pres_move.end_y < 0 || pres_move.end_y >= NUM_SQUARES_TALL)
	   	return false;
	
	//Basic Validity Check - No piece of same color exists in destination square
	if(dBoard[pres_move.end_y][pres_move.end_x].contains != 0 &&
	   dBoard[pres_move.end_y][pres_move.end_x].contains->isWhite == 
	   dBoard[pres_move.start_y][pres_move.start_x].contains->isWhite)
		 return false;
	if(pres_move.start_x == pres_move.end_x && pres_move.start_y == pres_move.end_y)
		return false;
	
	//Individual Move Checks - Each piece of that type 
	switch(dBoard[pres_move.start_y][pres_move.start_x].contains->type)
	{
		case pawn: return isValidPawnMove(dBoard, pres_move, last_move, pics, allowspecial);
		case rook: return isValidRookMove(dBoard, pres_move);
		case knight: return isValidKnightMove(pres_move);
		case bishop: return isValidBishopMove(dBoard, pres_move);
		case king: return isValidKingMove(dBoard, pres_move, allowspecial);
		case queen: return isValidQueenMove(dBoard, pres_move);
		default: return false;
	}
}


bool isValidPawnMove(square_t **dBoard, move_t pres_move, move_t last_move, picture_t pics, bool allowenpass)
{
	//For White Check to see if moving forward (1 or 2) and no piece in front, if so, return true
	if(pres_move.start_y == 6 && pres_move.end_y ==4 && !(dBoard[pres_move.end_y][pres_move.end_x].contains)
	   && pres_move.start_x == pres_move.end_x && dBoard[pres_move.start_y][pres_move.start_x].contains->isWhite
	   && !(dBoard[pres_move.end_y+1][pres_move.end_x].contains))
	    return true;
	
	//For Black Check to see if moving forward (1 or 2) and no piece in front, if so, return true
	if(pres_move.start_y == 1 && pres_move.end_y == 3 && !(dBoard[pres_move.end_y][pres_move.end_x].contains)
	   && pres_move.start_x == pres_move.end_x && !(dBoard[pres_move.start_y][pres_move.start_x].contains->isWhite)
	   && !(dBoard[pres_move.end_y-1][pres_move.end_x].contains))
	    return true;
	
	//Check standard forward moves
	if(!(dBoard[pres_move.end_y][pres_move.end_x].contains) && pres_move.start_x == pres_move.end_x &&
		((dBoard[pres_move.start_y][pres_move.start_x].contains->isWhite && pres_move.start_y - pres_move.end_y == 1) ||
	    (!(dBoard[pres_move.start_y][pres_move.start_x].contains->isWhite) && pres_move.start_y - pres_move.end_y == -1)))
	{
	 	//For Queen
	   	if(pres_move.end_y == 0 || pres_move.end_y == 7)
	 	{
	 		dBoard[pres_move.start_y][pres_move.start_x].contains->type = queen;
	 		
	 		if(dBoard[pres_move.start_y][pres_move.start_x].contains->isWhite)
	 			dBoard[pres_move.start_y][pres_move.start_x].contains->pic = pics.wqueen;
	 		else
	 			dBoard[pres_move.start_y][pres_move.start_x].contains->pic = pics.bqueen;
		}
	 	return true;
	}
	
	//Check to see if moving diagonally 1 and capturing, if so return true
	if(abs(pres_move.start_x - pres_move.end_x) == 1 && dBoard[pres_move.end_y][pres_move.end_x].contains &&
	   ((dBoard[pres_move.start_y][pres_move.start_x].contains->isWhite && pres_move.start_y - pres_move.end_y == 1) ||
	   (!(dBoard[pres_move.start_y][pres_move.start_x].contains->isWhite) && pres_move.start_y - pres_move.end_y == -1)))
	{	
	 	//For Queen
	  	if(pres_move.end_y == 7 || pres_move.end_y == 0)
	 	{
		 	dBoard[pres_move.start_y][pres_move.start_x].contains->type = queen;
			
	 		if(dBoard[pres_move.start_y][pres_move.start_x].contains->isWhite)
	 			dBoard[pres_move.start_y][pres_move.start_x].contains->pic = pics.wqueen;
	 		else
	 			dBoard[pres_move.start_y][pres_move.start_x].contains->pic = pics.bqueen;
		}
		 	  	
	 	return true;
	}
			
	//Deal with en passant
	if(abs(last_move.start_y - last_move.end_y) == 2 && last_move.start_x == pres_move.end_x && allowenpass &&
	   dBoard[last_move.end_y][last_move.end_x].contains->type == pawn && last_move.end_y == pres_move.start_y &&
	   ((dBoard[pres_move.start_y][pres_move.start_x].contains->isWhite && pres_move.start_y - pres_move.end_y == 1) ||
	   (!(dBoard[pres_move.start_y][pres_move.start_x].contains->isWhite) && pres_move.start_y - pres_move.end_y == -1)))
	{
		free(dBoard[last_move.end_y][last_move.end_x].contains);
		dBoard[last_move.end_y][last_move.end_x].contains = 0;
		drawSquare(dBoard, last_move.end_x, last_move.end_y);
		return true;
	}
		
	return false;
}


bool isValidRookMove(square_t **dBoard, move_t move)
{
	int i;
	
	//Check y Move
	if(move.start_x == move.end_x)
	{
		for(i= ((move.start_y < move.end_y) ? move.start_y : move.end_y)+1; i < move.end_y || i < move.start_y; i++)
			if(dBoard[i][move.end_x].contains)
				return false;
		return true;
	}
					
	//Check x Move
	if(move.start_y == move.end_y)
	{
		for(i= ((move.start_x < move.end_x) ? move.start_x : move.end_x)+1; i < move.end_x || i < move.start_x; i++)
			if(dBoard[move.end_y][i].contains)
				return false;
		return true;
	}
	
	return false;
}


bool isValidKnightMove(move_t move)
{
	//if x dif = 1 then y dif =2
	if(abs(move.start_x - move.end_x) == 1 && abs(move.start_y - move.end_y) ==2)
		return true;
	
	//if y dif = 1 then x dif ==2
	if(abs(move.start_x - move.end_x) == 2 && abs(move.start_y - move.end_y) ==1)
		return true;
	
	return false;
}


bool isValidBishopMove(square_t **dBoard, move_t move)
{
	int ly, lx, uy, ux, i, xcoord, ycoord;
	bool positive_slope;
	
	if(move.start_x == move.end_x || move.end_y == move.start_y)
		return false;
	
	if(abs((move.start_x - move.end_x)/(move.start_y-move.end_y)) != 1)
		return false;
	
	else if(move.start_y < move.end_y)
	{
		ly = move.start_y;
		lx = move.start_x;
		uy = move.end_y;
		ux = move.end_x;
		positive_slope = (move.start_x < move.end_x);
	}
	
	else
	{
		ly = move.end_y;
		lx = move.end_x;
		uy = move.start_y;
		ux = move.start_x;
		positive_slope = (move.start_x > move.end_x);
	}
	
	for(i=1; i < uy - ly; i++)
	{
		xcoord = positive_slope ? lx + i : lx - i;
		ycoord = ly + i;
		
		if(xcoord == -1 || xcoord == NUM_SQUARES_WIDE || ycoord == -1 || ycoord == NUM_SQUARES_TALL)
			return false;
		
		 if(dBoard[ycoord][xcoord].contains)
		 	return false;
	}
	
	return true;
}


bool isValidQueenMove(square_t **dBoard, move_t move)
{
	return (isValidBishopMove(dBoard, move) || isValidRookMove(dBoard, move));
}


bool isValidKingMove(square_t **dBoard, move_t move, bool allowcastle)
{
	if(abs(move.start_x - move.end_x) == 1 && abs(move.start_y - move.end_y) == 1)
		return true;
	if(abs(move.start_x - move.end_x) == 0 && abs(move.start_y - move.end_y) == 1)
		return true;
	if(abs(move.start_x - move.end_x) == 1 && abs(move.start_y - move.end_y) == 0)
		return true;
	
	if(!allowcastle)
		return false;
	
	//Deal with Castling left
	if(move.start_x - move.end_x == 2 && move.start_y == move.end_y  && move.start_x == 4 
	   && dBoard[move.end_y][0].contains->type == rook && 
	   !isCheck(dBoard, dBoard[move.start_y][move.start_x].contains->isWhite))
	{
		//Deal with White
		if(dBoard[move.start_y][move.start_x].contains->isWhite && move.start_y == 7 && 
		   !(dBoard[7][1].contains) && !(dBoard[7][2].contains) && !(dBoard[7][3].contains))
		{
			dBoard[7][3].contains = dBoard[7][0].contains;
			dBoard[7][0].contains = 0;
			drawSquare(dBoard, 0, 7);
			drawSquare(dBoard, 3, 7);
			return true;
		}
		
		//Deal with Black
		if(!(dBoard[move.start_y][move.start_x].contains->isWhite) && move.start_y == 0 && 
		   !(dBoard[0][1].contains) && !(dBoard[0][2].contains) && !(dBoard[0][3].contains))
		{
			dBoard[0][3].contains = dBoard[0][0].contains;
			dBoard[0][0].contains = 0;
			drawSquare(dBoard, 0, 0);
			drawSquare(dBoard, 3, 0);
			return true;
		}
	}
	
	//Castling Right
	if(move.start_x - move.end_x == -2 && move.start_y == move.end_y  && move.start_x == 4 
	   && dBoard[move.end_y][7].contains->type == rook && 
	   !isCheck(dBoard, dBoard[move.start_y][move.start_x].contains->isWhite))
	{
		//Deal with White
		if(dBoard[move.start_y][move.start_x].contains->isWhite && move.start_y == 7 && 
		   !(dBoard[7][6].contains) && !(dBoard[7][5].contains))
		{
			dBoard[7][5].contains = dBoard[7][7].contains;
			dBoard[7][7].contains = 0;
			drawSquare(dBoard, 7, 7);
			drawSquare(dBoard, 5, 7);
			return true;
		}
		
		//Deal with Black
		if(!(dBoard[move.start_y][move.start_x].contains->isWhite) && move.start_y == 0 && 
		   !(dBoard[0][6].contains) && !(dBoard[0][5].contains))
		{
			dBoard[0][5].contains = dBoard[0][7].contains;
			dBoard[0][7].contains = 0;
			drawSquare(dBoard, 7, 0);
			drawSquare(dBoard, 5, 0);
			return true;
		}
	}
	
	return false;
}


bool isCheck(square_t **dBoard, bool isWhite)
{
	int i, j, king_x, king_y;
	
	for(i=0; i < NUM_SQUARES_TALL; i++)
	{
		for(j=0; j < NUM_SQUARES_WIDE; j++)
		{
			if(dBoard[i][j].contains && dBoard[i][j].contains->type == king && 
			   dBoard[i][j].contains->isWhite == isWhite)
			{
				king_x = j;
				king_y = i;
				break;
			}
		}
		
		if(j != NUM_SQUARES_WIDE)
			break;
	}
	
	//Diagonal Check (Pawns, Kings if == 1), else Bishops and Queens (other color only)
	for(i=king_y+1, j=king_x+1; i >= 0 && j >= 0 && i < NUM_SQUARES_TALL && j < NUM_SQUARES_WIDE; i++, j++)
	{
		if(dBoard[i][j].contains)
		{
			if(i==king_y +1 && (dBoard[i][j].contains->type == pawn) && dBoard[i][j].contains->isWhite && !isWhite)
				return true;
			if((dBoard[i][j].contains->type == bishop || dBoard[i][j].contains->type == queen) && 
				dBoard[i][j].contains->isWhite != isWhite)
				return true;
			if(dBoard[i][j].contains->isWhite == isWhite)
				break;
		}
	}
	for(i=king_y+1, j=king_x-1; i >= 0 && j >= 0 && i < NUM_SQUARES_TALL && j < NUM_SQUARES_WIDE; i++, j--)
	{
		if(dBoard[i][j].contains)
		{
			if(i==king_y +1 && (dBoard[i][j].contains->type == pawn) && dBoard[i][j].contains->isWhite && !isWhite)
				return true;
			if((dBoard[i][j].contains->type == bishop || dBoard[i][j].contains->type == queen) && 
				dBoard[i][j].contains->isWhite != isWhite)
				return true;
			if(dBoard[i][j].contains->isWhite == isWhite)
				break;
		}
	}
	for(i=king_y-1, j=king_x+1; i >= 0 && j >= 0 && i < NUM_SQUARES_TALL && j < NUM_SQUARES_WIDE; i--, j++)
	{
		if(dBoard[i][j].contains)
		{
			if(i==king_y -1 && (dBoard[i][j].contains->type == pawn) && !(dBoard[i][j].contains->isWhite) && isWhite)
				return true;
			if((dBoard[i][j].contains->type == bishop || dBoard[i][j].contains->type == queen) && 
				dBoard[i][j].contains->isWhite != isWhite)
				return true;
			if(dBoard[i][j].contains->isWhite == isWhite)
				break;
		}
	}
	for(i=king_y-1, j=king_x-1; i >= 0 && j >= 0 && i < NUM_SQUARES_TALL && j < NUM_SQUARES_WIDE; i--, j--)
	{
		if(dBoard[i][j].contains)
		{
			if(i==king_y -1 && (dBoard[i][j].contains->type == pawn) && !(dBoard[i][j].contains->isWhite) && isWhite)
				return true;
			if((dBoard[i][j].contains->type == bishop || dBoard[i][j].contains->type == queen) && 
				dBoard[i][j].contains->isWhite != isWhite)
				return true;
			if(dBoard[i][j].contains->isWhite == isWhite)
				break;
		}
	}
	
	//Straight Check (Kings if == 1), else Rooks, Queens (other color only)
	for(i=king_y+1, j = king_x; i >= 0 && i < NUM_SQUARES_TALL; i++)
	{
		if(dBoard[i][j].contains)
		{
			if(dBoard[i][j].contains->isWhite == isWhite)
				break;
			if(dBoard[i][j].contains->type == rook || dBoard[i][j].contains->type == queen)
				return true;
		}
	}
	for(i=king_y-1, j = king_x; i >= 0 && i < NUM_SQUARES_TALL; i--)
	{
		if(dBoard[i][j].contains)
		{
			if(dBoard[i][j].contains->isWhite == isWhite)
				break;
			if(dBoard[i][j].contains->type == rook || dBoard[i][j].contains->type == queen)
				return true;
		}
	}
	for(i=king_y, j = king_x+1; j >= 0 && j < NUM_SQUARES_WIDE; j++)
	{
		if(dBoard[i][j].contains)
		{
			if(dBoard[i][j].contains->isWhite == isWhite)
				break;
			if(dBoard[i][j].contains->type == rook || dBoard[i][j].contains->type == queen)
				return true;
		}
	}
	for(i=king_y, j = king_x-1; j >= 0 && j < NUM_SQUARES_WIDE; j--)
	{
		if(dBoard[i][j].contains)
		{
			if(dBoard[i][j].contains->isWhite == isWhite)
				break;
			if(dBoard[i][j].contains->type == rook || dBoard[i][j].contains->type == queen)
				return true;
		}
	}
	
	//Check Knights of other color only
	i = king_y +1; j = king_x +2;
	if(i > 0 && j > 0 && i < NUM_SQUARES_TALL && j < NUM_SQUARES_WIDE && dBoard[i][j].contains &&
	   dBoard[i][j].contains->type == knight && dBoard[i][j].contains->isWhite != isWhite)
		return true;
	i = king_y +1; j = king_x -2;
	if(i > 0 && j > 0 && i < NUM_SQUARES_TALL && j < NUM_SQUARES_WIDE && dBoard[i][j].contains &&
	   dBoard[i][j].contains->type == knight && dBoard[i][j].contains->isWhite != isWhite)
	return true;
		i = king_y -1; j = king_x +2;
	if(i > 0 && j > 0 && i < NUM_SQUARES_TALL && j < NUM_SQUARES_WIDE && dBoard[i][j].contains &&
	   dBoard[i][j].contains->type == knight && dBoard[i][j].contains->isWhite != isWhite)
		return true;
	i = king_y -1; j = king_x -2;
	if(i > 0 && j > 0 && i < NUM_SQUARES_TALL && j < NUM_SQUARES_WIDE && dBoard[i][j].contains &&
	   dBoard[i][j].contains->type == knight && dBoard[i][j].contains->isWhite != isWhite)
		return true;
	i = king_y +2; j = king_x +1;
	if(i > 0 && j > 0 && i < NUM_SQUARES_TALL && j < NUM_SQUARES_WIDE && dBoard[i][j].contains &&
	   dBoard[i][j].contains->type == knight && dBoard[i][j].contains->isWhite != isWhite)
		return true;
	i = king_y +2; j = king_x -1;
	if(i > 0 && j > 0 && i < NUM_SQUARES_TALL && j < NUM_SQUARES_WIDE && dBoard[i][j].contains &&
	   dBoard[i][j].contains->type == knight && dBoard[i][j].contains->isWhite != isWhite)
		return true;
	i = king_y -2; j = king_x +1;
	if(i > 0 && j > 0 && i < NUM_SQUARES_TALL && j < NUM_SQUARES_WIDE && dBoard[i][j].contains &&
	   dBoard[i][j].contains->type == knight && dBoard[i][j].contains->isWhite != isWhite)
		return true;
	i = king_y -2; j = king_x -1;
	if(i > 0 && j > 0 && i < NUM_SQUARES_TALL && j < NUM_SQUARES_WIDE && dBoard[i][j].contains &&
	   dBoard[i][j].contains->type == knight && dBoard[i][j].contains->isWhite != isWhite)
		return true;
	
	return false;
}


//Checks to see if a proposed move results in check
bool moveChecks(square_t **dBoard, bool isWhite, move_t move)
{
	square_t **nBoard;
	int i, j;
	bool toreturn;
	
	nBoard = malloc(NUM_SQUARES_TALL * sizeof(square_t*));
	
	for(i=0; i < NUM_SQUARES_TALL; i++)
	{
		nBoard[i] = malloc(NUM_SQUARES_WIDE * sizeof(square_t));
		
		for(j=0; j < NUM_SQUARES_WIDE; j++)
			nBoard[i][j] = dBoard[i][j];
	}
	
	nBoard[move.end_y][move.end_x].contains = nBoard[move.start_y][move.start_x].contains;
	nBoard[move.start_y][move.start_x].contains = 0; 
	
	toreturn = isCheck(nBoard, isWhite);
	
	for(i=0; i < NUM_SQUARES_TALL; i++)
		free(nBoard[i]);
	free(nBoard);
	
	return toreturn;
}


//See if Color isWhite is mated
bool isMate(square_t **dBoard, bool isWhite)
{
	int king_x, king_y;
	
	if(!isCheck(dBoard, isWhite))
		return false;
	
	getKingCoords(dBoard, isWhite, &king_x, &king_y);
	
	if(!mateKingMove(dBoard, isWhite, king_x, king_y) || !mateOtherMove(dBoard, isWhite))
		return false;
	
	return true;
}



void getKingCoords(square_t ** dBoard, bool isWhite, int *x, int *y)
{
	int i, j;
	
	//Find king's x & y coords
	for(i=0; i < NUM_SQUARES_TALL; i++)
	{
		for(j=0; j < NUM_SQUARES_WIDE; j++)
		{
			if(dBoard[i][j].contains && dBoard[i][j].contains->type == king && 
			   dBoard[i][j].contains->isWhite == isWhite)
			{
				*x = j;
				*y = i;
				break;
			}
		}
		
		if(j != NUM_SQUARES_WIDE)
			break;
	}
}

//Returns true if a king cannot be moved to leave mate
bool mateKingMove(square_t **dBoard, bool isWhite, int king_x, int king_y)
{
	int i, j;
	move_t testMove;
	picture_t dummy;
	
	testMove.start_x = king_x;
	testMove.start_y = king_y;
	
	//See if moving king will escape mate
	for(i=-1; i <= 1; i++)
	{
		for(j=-1; j <= 1; j++)
		{	
			testMove.end_y = king_y + i;
			testMove.end_x = king_x + j;
			
			if(isValidMove(dBoard, testMove, testMove, dummy, false) && !moveChecks(dBoard, isWhite, testMove))
				return false;
		}
	}
	
	return true;
}

//Returns true if no way to block, otherwise returns false
bool mateOtherMove(square_t **dBoard, bool isWhite)
{
	int i, j;
	square_t **tBoard;
	
	//Find Stop Coords
	for(i=0; i < NUM_SQUARES_TALL; i++)
	{
		for(j=0; j < NUM_SQUARES_WIDE; j++)
		{
			if(!(dBoard[i][j].contains) || dBoard[i][j].contains->isWhite != isWhite)
			{
				tBoard = dupBoard(dBoard);
				tBoard[i][j].contains = malloc(sizeof(piece_t));
				tBoard[i][j].contains->isWhite =isWhite;
				tBoard[i][j].contains->type = pawn;
				
				if(!isCheck(tBoard, isWhite) && blockPossible(dBoard, isWhite, j, i))
					return false;
				
				free(tBoard[i][j].contains);
				delBoard(tBoard);
			}
		}
	}
	
	return true;
}


bool blockPossible(square_t **dBoard, bool isWhite, int block_x, int block_y)
{
	int i, j;
	move_t testMove;
	picture_t dummy;
	
	testMove.end_y = block_y;
	testMove.end_x = block_x;
	
	for(i=0; i < NUM_SQUARES_TALL; i++)
	{
		for(j=0; j < NUM_SQUARES_WIDE; j++)
		{
			if(dBoard[i][j].contains && dBoard[i][j].contains->isWhite == isWhite && dBoard[i][j].contains->type != king)
			{
				testMove.start_x = j;
				testMove.start_y = i;
				
				if(isValidMove(dBoard, testMove, testMove, dummy, false))
					return true;
			}
		}
	}
	
	return false;
}


square_t **dupBoard(square_t **dBoard)
{
	int i, j;
	square_t **tBoard;
	
	tBoard = malloc(NUM_SQUARES_TALL * sizeof(square_t *));
	
	for(i=0; i < NUM_SQUARES_TALL; i++)
	{
		tBoard[i] = malloc(NUM_SQUARES_WIDE * sizeof(square_t));
		
		for(j=0; j < NUM_SQUARES_WIDE; j++)
			tBoard[i][j] = dBoard[i][j];
	}
	
	return tBoard;
}

void delBoard(square_t **tBoard)
{

	int i;
	
	for(i=0; i < NUM_SQUARES_TALL; i++)
		free(tBoard[i]);
	free(tBoard);
}